-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
--Mandatory controller parameters
M.type = "auxiliary"
M.relevantDevice = nil

local min = math.min
local max = math.max
local abs = math.abs

local avToRPM = 9.549296596425384

local motorTop = nil
local motorBottom = nil
local targetRPMRatio = 0
local invMotorTopMaxAV = 0
local invMotorBottomMaxAV = 0
local maxRPM = 0
local rollerTop = nil
local rollerBottom = nil

local function updateGFX(dt)

  electrics.values.arm_A = (electrics.values.arm_A + electrics.values.arm_A_input * dt * 0.5)
  electrics.values.arm_B = (electrics.values.arm_B + electrics.values.arm_B_input * dt * 1)
  electrics.values.arm_C = (electrics.values.arm_C + electrics.values.arm_C_input * dt * 0.5)

  targetRPMRatio = min(max(targetRPMRatio + (electrics.values.targetRPMRatioIncrease * dt * 0.5) - (electrics.values.targetRPMRatioDecrease * dt * 0.5), 0), 1)

  local motorTopAV = abs(motorTop.outputAV1)
  local motorBottomAV = abs(motorBottom.outputAV1)
  local rpmRatioTop = min(max(motorTopAV * invMotorTopMaxAV, 0), 1)
  local rpmRatioBottom = min(max(motorBottomAV * invMotorBottomMaxAV, 0), 1)
  local isWrongDirectionTop = motorTop.outputAV1 * motorTop.motorDirection < 0
  local isWrongDirectionBottom = motorBottom.outputAV1 * motorBottom.motorDirection < 0

  local rpmErrorTop = targetRPMRatio - rpmRatioTop
  local rpmErrorBottom = targetRPMRatio - rpmRatioBottom

  local throttleTop = isWrongDirectionTop and 0 or min(max(rpmErrorTop * 10, 0), 1)
  local throttleBottom = isWrongDirectionBottom and 0 or min(max(rpmErrorBottom * 10, 0), 1)

  local brakeTop = isWrongDirectionTop and 1 or min(max(rpmErrorTop * -10, 0), 1)
  local brakeBottom = isWrongDirectionBottom and 1 or min(max(rpmErrorBottom * -10, 0), 1)
  if targetRPMRatio == 0 then
    if rpmErrorTop < 0.1 then
      brakeTop = 1
      throttleTop = 0
    end
    if rpmErrorBottom < 0.1 then
      brakeBottom = 1
      throttleBottom = 0
    end
  end

  rollerTop.desiredBrakingTorque = brakeTop * rollerTop.brakeTorque
  rollerBottom.desiredBrakingTorque = brakeBottom * rollerBottom.brakeTorque
  electrics.values.throttleTop = throttleTop
  electrics.values.throttleBottom = throttleBottom

end

local function toggleDirection()
  motorTop.motorDirection = motorTop.motorDirection * -1
  motorBottom.motorDirection = motorBottom.motorDirection * -1
end

local function liftA(value)
  electrics.values.arm_A_input = value
end

local function liftB(value)
  electrics.values.arm_B_input = value
end

local function liftC(value)
  electrics.values.arm_C_input = value
end

local function init(jbeamData)
  motorTop = powertrain.getDevice("motorTop")
  motorBottom = powertrain.getDevice("motorBottom")
  invMotorTopMaxAV = 1 / motorTop.maxAV
  invMotorBottomMaxAV = 1 / motorBottom.maxAV
  maxRPM = (motorTop.maxAV + motorBottom.maxAV) * 0.5 * avToRPM
  targetRPMRatio = 0
  electrics.values.targetRPMRatioIncrease = 0
  electrics.values.targetRPMRatioDecrease = 0

  for k, v in pairs(wheels.wheelRotators) do
    if v.name == "roller_U" then
      rollerTop = wheels.wheelRotators[k]
    end
    if v.name == "roller_D" then
      rollerBottom = wheels.wheelRotators[k]
    end
  end

  electrics.values.arm_A = 0
  electrics.values.arm_B = 0
  electrics.values.arm_C = 0
  electrics.values.arm_A_input = 0
  electrics.values.arm_B_input = 0
  electrics.values.arm_C_input = 0

end

M.init = init
M.updateGFX = updateGFX
M.toggleDirection = toggleDirection
M.liftA = liftA
M.liftB = liftB
M.liftC = liftC

return M
